/****** Zelda Game ***********************************************
The object of this game is to move the character around the screen
to collect the jewels (rupees) and avoid the blocks scrolling along
the screen.  Hitting a block with take away health while collecting
jewels will restore health (red), slow the blocks down (blue), or 
give more points (green).  This version is controlled by the mouse.

ME 333 - Mechatronics
HaptiLink Group
Philip Dames, Yang Bai, Megan Welker
3/14/2010
*****************************************************************/

// Initialize arrays to keep track of blocks on screen
ArrayList blocks;
int blockWidth = 100;

// Initialize parameters to keep track of rupees on screen
ArrayList rupees;
PImage  brupee  =  loadImage("blue rupee.png");
PImage  rrupee  =  loadImage("red rupee.png");
PImage  grupee  =  loadImage("green rupee.png");
int  rupeeWidth  =  15;
int  rupeeHeight  =  27;

// Define size of zelda image
int zeldaH = 31;
int zeldaW = 33;

// Initialize game parameters
int hp = 5; // hit point
int speed0 = 5; // initial speed
int groundHeight = 50;
int bSpeed = speed0; // base speed
int maxSpeed = 10; // max speed
int score  =  0; //records the score
boolean flag = false; // goes true when get hurt

//int  outflag  =  0;   //denotes number of states to keep outside
PFont fontA;
PFont  fontB;

// Initialize position variables
int Y = 0; // plot y value

// setup() function
void setup() {
  //set up font
  fontA  =  loadFont("Aparajita-Bold-68.vlw");
  textFont(fontA,32);
  fontB  =  loadFont("AgencyFB-Bold-48.vlw");

  //font setup
  size(800,500);
  frameRate(30); 
  println("Program Started!"); // print some text to the debug window

  // Add first block and rupee
  blocks = new ArrayList();
  rupees = new  ArrayList();
  blocks.add(new block(width, height-groundHeight-100, blockWidth, 100, speed0, 255, 0, 0));
  rupees.add(new rupee(width,height-groundHeight-27,15,27,speed0,1));
}


// draw() function
void draw() 
{
  if(frameCount%1 == 0) { // can be used to adjust how many frames are drawn if game is choppy
    //generate new blocks and new rupees when all blocks are removed
    if(blocks.size()==0) {
      blocks.add(new block(width, height-groundHeight-100, blockWidth, 100, bSpeed, 255, 0, 0));
    }
    if(rupees.size()==0) {
      rupees.add(new  rupee(width,height-groundHeight-27,15,27,speed0,1));
    }

    //zelda CANNOT cheat by hiding in the grass!!!!!
    if(mouseY >= height-groundHeight-zeldaH) {
      Y = height-groundHeight-zeldaH;
    }
    else {
      Y = mouseY;
    }
    //above codes get x and y position of zelda  

    // speed up as play longer
    //bSpeed = speed0 + millis()/5000;
    if(bSpeed > maxSpeed) {
      bSpeed = maxSpeed;
    }

    // draw background and ground
    background(100, 180, 255); // sky blue background
    fill(85, 200, 50); // green grass
    rect(0, height-groundHeight, width, groundHeight);
    PImage zelda = loadImage("zelda.png");
    image(zelda,mouseX,Y);

    // if no game over, then draw objects
    if(!flag) {
      // draw the rupees
      for(int  k = rupees.size()-1;  k >= 0;  k--) {
        rupee temp = (rupee) rupees.get(k);
        temp.update(bSpeed);

        // draw next rupee if last one has moved far enough
        if(k == rupees.size()-1) {
          int  randrupees  =  int(random(15,25));
          if(temp.getx() < width - randrupees*27) {
            int randoms= int(random(1,4));
            if(randoms <= 3) {   //increase the possiblity that no rupees are generated
              rupees.add(new rupee(width, height-groundHeight-27, 15, 27, bSpeed, randoms)  );
            }
          }
        }
        // remove picture if has been picked up
        if(temp.overlap(mouseX,Y)) { rupees.remove(k);}
        // remove picture if has moved off screen
        if(temp.getx() <= -rupeeWidth) { rupees.remove(k);}
      }
      //end of for(k) loop, generates rupees


      //start of for(i) loop, generates blocks
      for(int i = blocks.size()-1; i >= 0; i--) {
        block temp = (block) blocks.get(i);
        temp.update(bSpeed);
        
        // create next block if last one has moved far enough
        if(i == blocks.size()-1) {
          int  randblocks  =  int(random(5,9));
          if(temp.getx() < width - randblocks*blockWidth) {
            int randH = int(random(200));
            blocks.add(new block(width, height-groundHeight-randH, blockWidth, randH, bSpeed, 255, 0, 0));
          }
        }
        // remove block and take actions zelda has run into it
        if(temp.overlap(mouseX,Y)) {//given top left position of two objects, check if overlap happens
          hp--;
          judge(hp);
          blocks.remove(i);
        }
        // remove block if it has moved off the screen
        if(temp.getx() <= -blockWidth) { blocks.remove(i);}
      }// end of for(i) loop
    }
    gm(flag); // check if game over
    fillhp(hp); // draw hp bar
  }
}


// define a class for blocks
class block {
  int xpos, ypos, xsize, ysize, speed, R, G, B;
  block(int x, int y, int w, int h, int v, int Red, int Green, int Blue){
    xpos = x; // location to draw from
    ypos = y;
    xsize = w; // size of block
    ysize = h;
    speed = v; // speed
    R = Red; // color of block
    G = Green;
    B = Blue;
  }
  //update location
  void update(int v) {
    xpos -= speed;
    fill(R, G, B);
    rect(xpos, ypos, xsize, ysize);
    speed = v;
  }
  // get current location
  int getx() {
    return xpos;
  }

  int gety() {
    return ypos;
  }
  // check if character is inside of block
  boolean overlap(int x1, int y1) {
    if((xpos <= x1+zeldaW)&&(x1+zeldaW <= xpos+blockWidth)&&(ypos <= y1+zeldaH)&&(y1+zeldaH <= ypos+ysize)) {
      return true;//bottom right corner
    }
    if((xpos <= x1)&&(x1 <= xpos+blockWidth)&&(ypos <= y1+zeldaH)&&(y1+zeldaH <= ypos+ysize)) {
      return true;//bottom left corner
    }
    if((xpos<=x1)&&(x1 <= xpos+blockWidth)&&(ypos <= y1)&&(y1 <= ypos+ysize)) {
      return true;//top left corner
    }
    if((xpos <= x1+zeldaW)&&(x1+zeldaW <= xpos+blockWidth)&&(ypos <= y1)&&(y1 <= ypos+ysize)) {
      return true;//top right corner
    }
    else {
      return false;
    }
  }
}


//mouselick function to reset game
void mouseClicked() {
  flag = false;
  hp = 5;
  score = 0;//reset score
}


//define rupee class 
class  rupee {
  int  xpos, ypos, xsize, ysize, speed, type;
  PImage  rup;
  rupee(int  x, int  y, int  w, int  h, int  v, int  ran) {
    xpos  =  x; // location to draw from
    ypos  =  y;
    xsize  =  w; // size
    ysize  =  h;
    speed  =  v; // speed
    type  =  ran; // color
  } 
  // draw on screen
  void update(int v) {
    xpos -= speed;
    if(type == 1) {
      PImage  rup  =  loadImage("blue rupee.png");
      image(rup, xpos, ypos);
    }
    if(type == 2) {
      PImage  rup  =  loadImage("green rupee.png");
      image(rup, xpos, ypos);
    }
    if(type == 3) {
      PImage  rup  =  loadImage("red rupee.png");
      image(rup, xpos, ypos);
    }
    speed = v;
  }
  // get current location and type
  int getx() {
    return xpos;
  }

  int gety() {
    return ypos;
  }

  int  gettype() {
    return type;    //reture the input rupee type
  }
  // check if character has run into rupee
  boolean overlap(int x1, int y1) {//collect rupees
    boolean collision = false;

    // Check to see if there is a collision
    if((xpos <= x1+zeldaW)&&(x1+zeldaW <= xpos+rupeeWidth)&&(ypos <= y1+zeldaH)&&(y1+zeldaH <= ypos+ysize)) {
      collision = true;//bottom right corner
    }
    if((xpos <= x1)&&(x1 <= xpos+rupeeWidth)&&(ypos <= y1+zeldaH)&&(y1+zeldaH <= ypos+ysize)) {
      collision = true;//bottom left corner
    }
    if((xpos <= x1)&&(x1 <= xpos+rupeeWidth)&&(ypos <= y1)&&(y1 <= ypos+ysize)) {
      collision = true;//top left corner
    }
    if((xpos <= x1+zeldaW)&&(x1+zeldaW <= xpos+rupeeWidth)&&(ypos <= y1)&&(y1 <= ypos+ysize)) {
      collision = true;//top right corner
    }

    // If there is a collsion, take the appropriate action
    if(collision) {
      if(type == 1) { //blue rupees
        speed0 --;    //restore beginning speed
        score++;
      }
      if(type == 2) { //green rupees
        score += 2;              //score increased
      }
      if((type == 3)&&(hp <= 5)) { //red rupees
        hp++;
        score++;
      }
    }
    return collision;
  }
}

//game over function
void gm(boolean flag) {
  if(flag == true) {
    flag = false;
    bSpeed = speed0;
    fill(0);
    text("GAME OVER", 320, 200);
    text("SCORE    "+score, 320, 250);
  }
  else {
    textFont(fontB,31);
    fill(212, 102, 1);
    text("SCORE    "+score, 650, 50);
  }
}


//define fillhp function that changes HP bar
void fillhp(int bar) {
  noStroke();
  if(bar==6) {  
    fill(41,255,204);
    rect(20,20,100,20);
  }
  if(bar==5) {
    fill(51,255,0);
    rect(20,20,100,20);
  }
  if(bar==4) {  
    fill(153,255,51);
    rect(20,20,80,20);
  }
  if(bar==3) {
    fill(255,255,0);
    rect(20,20,60,20);
  }
  if(bar==2) {
    fill(204,153,51);
    rect(20,20,40,20);
  }
  if(bar==1) {  
    fill(204,0,0);
    rect(20,20,20,20);
  }   
}


// check if there is a game over
void judge(int indi) {
  if(indi == 0) {
    flag = true;
  }
  else {
    flag = false;
  }
}

